diff --git a/extern/sdl_mixer/include/SDL_mixer.h b/extern/sdl_mixer/include/SDL_mixer.h
index 009d8a2da6..6acf7a9aba 100644
--- a/extern/sdl_mixer/include/SDL_mixer.h
+++ b/extern/sdl_mixer/include/SDL_mixer.h
@@ -567,6 +567,12 @@ extern DECLSPEC int SDLCALL Mix_Volume(int channel, int volume);
 extern DECLSPEC int SDLCALL Mix_VolumeChunk(Mix_Chunk *chunk, int volume);
 extern DECLSPEC int SDLCALL Mix_VolumeMusic(int volume);
 
+/* Set the master volume.
+   This did not affect the variables for music, channel or chunck volume.
+   If the specified volume is -1, just return the current master volume.
+*/
+extern DECLSPEC int SDLCALL Mix_MasterVolume(int volume);
+
 /* Halt playing of a particular channel */
 extern DECLSPEC int SDLCALL Mix_HaltChannel(int channel);
 extern DECLSPEC int SDLCALL Mix_HaltGroup(int tag);
diff --git a/extern/sdl_mixer/src/mixer.c b/extern/sdl_mixer/src/mixer.c
index ba2b407b12..413b962cd3 100644
--- a/extern/sdl_mixer/src/mixer.c
+++ b/extern/sdl_mixer/src/mixer.c
@@ -43,6 +43,7 @@
 #define CREA        0x61657243      /* "Crea" */
 
 static int audio_opened = 0;
+static int master_volume = MIX_MAX_VOLUME;
 static SDL_AudioSpec mixer;
 static SDL_AudioDeviceID audio_device;
 
@@ -260,7 +261,7 @@ static void SDLCALL
 mix_channels(void *udata, Uint8 *stream, int len)
 {
     Uint8 *mix_input;
-    int i, mixable, volume = MIX_MAX_VOLUME;
+    int i, mixable, volume = Mix_MasterVolume(-1);
     Uint32 sdl_ticks;
 
 #if SDL_VERSION_ATLEAST(1, 3, 0)
@@ -307,7 +308,7 @@ mix_channels(void *udata, Uint8 *stream, int len)
                 int remaining = len;
                 while (mix_channel[i].playing > 0 && index < len) {
                     remaining = len - index;
-                    volume = (mix_channel[i].volume*mix_channel[i].chunk->volume) / MIX_MAX_VOLUME;
+                    volume = (volume*(mix_channel[i].volume*mix_channel[i].chunk->volume)) / (MIX_MAX_VOLUME*MIX_MAX_VOLUME);
                     mixable = mix_channel[i].playing;
                     if (mixable > remaining) {
                         mixable = remaining;
@@ -1605,6 +1606,21 @@ void Mix_UnlockAudio(void)
     SDL_UnlockAudioDevice(audio_device);
 }
 
+int Mix_MasterVolume(int volume)
+{
+    int prev_volume;
+
+    prev_volume = master_volume;
+    if (volume < 0) {
+        return prev_volume;
+    }
+    if (volume > SDL_MIX_MAXVOLUME) {
+        volume = SDL_MIX_MAXVOLUME;
+    }
+    master_volume = volume;
+    return(prev_volume);
+}
+
 /* end of mixer.c ... */
 
 /* vi: set ts=4 sw=4 expandtab: */
diff --git a/extern/sdl_mixer/src/music.c b/extern/sdl_mixer/src/music.c
index a09373f8dc..f02194537e 100644
--- a/extern/sdl_mixer/src/music.c
+++ b/extern/sdl_mixer/src/music.c
@@ -223,14 +223,14 @@ void SDLCALL music_mixer(void *udata, Uint8 *stream, int len)
         /* Handle fading */
         if (music_playing->fading != MIX_NO_FADING) {
             if (music_playing->fade_step++ < music_playing->fade_steps) {
-                int volume;
+                int volume = Mix_MasterVolume(-1);
                 int fade_step = music_playing->fade_step;
                 int fade_steps = music_playing->fade_steps;
 
                 if (music_playing->fading == MIX_FADING_OUT) {
-                    volume = (music_volume * (fade_steps-fade_step)) / fade_steps;
+                    volume = (volume * (music_volume * (fade_steps-fade_step))) / (fade_steps * MIX_MAX_VOLUME);
                 } else { /* Fading in */
-                    volume = (music_volume * fade_step) / fade_steps;
+                    volume = (volume * (music_volume * fade_step)) / (fade_steps * MIX_MAX_VOLUME);
                 }
                 music_internal_volume(volume);
             } else {
